@@ -133,8 +133,9 @@ DEF_ENUM_FLAGS_TYPE (enum type_instance_flag_value, type_instance_flags);
   ((t)->dyn_prop (DYN_PROP_VARIANT_PARTS) != nullptr)
 
 /* * True if this type has a dynamic length.  */
-#define TYPE_HAS_DYNAMIC_LENGTH(t) \
-  ((t)->dyn_prop (DYN_PROP_BYTE_SIZE) != nullptr)
+#define TYPE_HAS_DYNAMIC_LENGTH(t)			\
+  (((t)->dyn_prop (DYN_PROP_BYTE_SIZE) != nullptr)	\
+   || ((t)->dyn_prop (DYN_PROP_BIT_SIZE) != nullptr))
 
 /* * Instruction-space delimited type.  This is for Harvard architectures
    which have separate instruction and data address spaces (and perhaps
@@ -456,6 +457,9 @@ enum dynamic_prop_node_kind
 
   /* A property holding the size of the type.  */
   DYN_PROP_BYTE_SIZE,
+
+  /* A property holding the size of the type, in bits.  */
+  DYN_PROP_BIT_SIZE,
 };
 
 /* * List for dynamic type attributes.  */
@@ -1931,26 +1935,6 @@ extern unsigned type_align (struct type *);
    space in struct type.  */
 extern bool set_type_align (struct type *, ULONGEST);
 
-/* Property accessors for the type data location.  */
-#define TYPE_DATA_LOCATION(thistype) \
-  ((thistype)->dyn_prop (DYN_PROP_DATA_LOCATION))
-#define TYPE_DATA_LOCATION_BATON(thistype) \
-  TYPE_DATA_LOCATION (thistype)->data.baton
-#define TYPE_DATA_LOCATION_ADDR(thistype) \
-  (TYPE_DATA_LOCATION (thistype)->const_val ())
-#define TYPE_DATA_LOCATION_KIND(thistype) \
-  (TYPE_DATA_LOCATION (thistype)->kind ())
-#define TYPE_DYNAMIC_LENGTH(thistype) \
-  ((thistype)->dyn_prop (DYN_PROP_BYTE_SIZE))
-
-/* Property accessors for the type allocated/associated.  */
-#define TYPE_ALLOCATED_PROP(thistype) \
-  ((thistype)->dyn_prop (DYN_PROP_ALLOCATED))
-#define TYPE_ASSOCIATED_PROP(thistype) \
-  ((thistype)->dyn_prop (DYN_PROP_ASSOCIATED))
-#define TYPE_RANK_PROP(thistype) \
-  ((thistype)->dyn_prop (DYN_PROP_RANK))
-
 /* C++ */
 
 #define TYPE_SELF_TYPE(thistype) internal_type_self_type (thistype)
@@ -2157,8 +2141,8 @@ struct builtin_type
      explicitly said that pointers to functions and pointers to data
      are not interconvertible --- that is, you can't cast a function
      pointer to void * and back, and expect to get the same value.
-     However, all function pointer types are interconvertible, so void
-     (*) () can server as a generic function pointer.  */
+     However, all function pointer types are interconvertible, so
+     `void (*) ()` can serve as a generic function pointer.  */
 
   struct type *builtin_func_ptr = nullptr;
 
@@ -2331,6 +2315,11 @@ public:
      if type-smashing was selected at construction.  */
   type *new_type (enum type_code code, int bit, const char *name);
 
+  /* Create a copy of TYPE on the desired obstack.  This is
+     incompatible with the "SMASH" kind; this is verified using an
+     assert.  */
+  type *copy_type (const type *type);
+
   /* Return the architecture associated with this allocator.  This
      comes from whatever object was supplied to the constructor.  */
   gdbarch *arch ();
@@ -2855,6 +2844,10 @@ using copied_types_hash_t = gdb::unordered_map<type *, type *>;
 extern struct type *copy_type_recursive (struct type *type,
 					 copied_types_hash_t &copied_types);
 
+/* Make a copy of the given TYPE, except that the pointer & reference
+   types are not preserved.  The new type is allocated using the same
+   storage as TYPE.  */
+
 extern struct type *copy_type (const struct type *type);
 
 extern bool types_equal (struct type *, struct type *);
